# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.conflict_error import ConflictError
from ..errors.not_found_error import NotFoundError
from ..types.dashboard_page_public import DashboardPagePublic
from ..types.dashboard_public import DashboardPublic
from ..types.json_node_public import JsonNodePublic
from ..types.json_node_write import JsonNodeWrite
from .types.dashboard_update_public_scope import DashboardUpdatePublicScope
from .types.dashboard_update_public_type import DashboardUpdatePublicType
from .types.dashboard_write_scope import DashboardWriteScope
from .types.dashboard_write_type import DashboardWriteType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDashboardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def find_dashboards(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        sorting: typing.Optional[str] = None,
        filters: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DashboardPagePublic]:
        """
        Find dashboards in a workspace

        Parameters
        ----------
        page : typing.Optional[int]

        size : typing.Optional[int]

        name : typing.Optional[str]

        sorting : typing.Optional[str]

        filters : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DashboardPagePublic]
            Dashboard page
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/private/dashboards",
            method="GET",
            params={
                "page": page,
                "size": size,
                "name": name,
                "sorting": sorting,
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DashboardPagePublic,
                    parse_obj_as(
                        type_=DashboardPagePublic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_dashboard(
        self,
        *,
        name: str,
        config: JsonNodeWrite,
        type: typing.Optional[DashboardWriteType] = OMIT,
        scope: typing.Optional[DashboardWriteScope] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DashboardPublic]:
        """
        Create a new dashboard in a workspace

        Parameters
        ----------
        name : str

        config : JsonNodeWrite

        type : typing.Optional[DashboardWriteType]

        scope : typing.Optional[DashboardWriteScope]

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DashboardPublic]
            Created
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/private/dashboards",
            method="POST",
            json={
                "name": name,
                "type": type,
                "scope": scope,
                "description": description,
                "config": config,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DashboardPublic,
                    parse_obj_as(
                        type_=DashboardPublic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_dashboard_by_id(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DashboardPublic]:
        """
        Get dashboard by id

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DashboardPublic]
            Dashboard resource
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/private/dashboards/{jsonable_encoder(dashboard_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DashboardPublic,
                    parse_obj_as(
                        type_=DashboardPublic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_dashboard(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Delete dashboard by id

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/private/dashboards/{jsonable_encoder(dashboard_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_dashboard(
        self,
        dashboard_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        type: typing.Optional[DashboardUpdatePublicType] = OMIT,
        scope: typing.Optional[DashboardUpdatePublicScope] = OMIT,
        description: typing.Optional[str] = OMIT,
        config: typing.Optional[JsonNodePublic] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DashboardPublic]:
        """
        Update dashboard by id. Partial updates are supported - only provided fields will be updated.

        Parameters
        ----------
        dashboard_id : str

        name : typing.Optional[str]

        type : typing.Optional[DashboardUpdatePublicType]

        scope : typing.Optional[DashboardUpdatePublicScope]

        description : typing.Optional[str]

        config : typing.Optional[JsonNodePublic]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DashboardPublic]
            Updated dashboard
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/private/dashboards/{jsonable_encoder(dashboard_id)}",
            method="PATCH",
            json={
                "name": name,
                "type": type,
                "scope": scope,
                "description": description,
                "config": config,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DashboardPublic,
                    parse_obj_as(
                        type_=DashboardPublic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_dashboards_batch(
        self, *, ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Delete dashboards batch

        Parameters
        ----------
        ids : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/private/dashboards/delete-batch",
            method="POST",
            json={
                "ids": ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDashboardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def find_dashboards(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        sorting: typing.Optional[str] = None,
        filters: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DashboardPagePublic]:
        """
        Find dashboards in a workspace

        Parameters
        ----------
        page : typing.Optional[int]

        size : typing.Optional[int]

        name : typing.Optional[str]

        sorting : typing.Optional[str]

        filters : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DashboardPagePublic]
            Dashboard page
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/private/dashboards",
            method="GET",
            params={
                "page": page,
                "size": size,
                "name": name,
                "sorting": sorting,
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DashboardPagePublic,
                    parse_obj_as(
                        type_=DashboardPagePublic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_dashboard(
        self,
        *,
        name: str,
        config: JsonNodeWrite,
        type: typing.Optional[DashboardWriteType] = OMIT,
        scope: typing.Optional[DashboardWriteScope] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DashboardPublic]:
        """
        Create a new dashboard in a workspace

        Parameters
        ----------
        name : str

        config : JsonNodeWrite

        type : typing.Optional[DashboardWriteType]

        scope : typing.Optional[DashboardWriteScope]

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DashboardPublic]
            Created
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/private/dashboards",
            method="POST",
            json={
                "name": name,
                "type": type,
                "scope": scope,
                "description": description,
                "config": config,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DashboardPublic,
                    parse_obj_as(
                        type_=DashboardPublic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_dashboard_by_id(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DashboardPublic]:
        """
        Get dashboard by id

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DashboardPublic]
            Dashboard resource
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/private/dashboards/{jsonable_encoder(dashboard_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DashboardPublic,
                    parse_obj_as(
                        type_=DashboardPublic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_dashboard(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete dashboard by id

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/private/dashboards/{jsonable_encoder(dashboard_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_dashboard(
        self,
        dashboard_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        type: typing.Optional[DashboardUpdatePublicType] = OMIT,
        scope: typing.Optional[DashboardUpdatePublicScope] = OMIT,
        description: typing.Optional[str] = OMIT,
        config: typing.Optional[JsonNodePublic] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DashboardPublic]:
        """
        Update dashboard by id. Partial updates are supported - only provided fields will be updated.

        Parameters
        ----------
        dashboard_id : str

        name : typing.Optional[str]

        type : typing.Optional[DashboardUpdatePublicType]

        scope : typing.Optional[DashboardUpdatePublicScope]

        description : typing.Optional[str]

        config : typing.Optional[JsonNodePublic]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DashboardPublic]
            Updated dashboard
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/private/dashboards/{jsonable_encoder(dashboard_id)}",
            method="PATCH",
            json={
                "name": name,
                "type": type,
                "scope": scope,
                "description": description,
                "config": config,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DashboardPublic,
                    parse_obj_as(
                        type_=DashboardPublic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_dashboards_batch(
        self, *, ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete dashboards batch

        Parameters
        ----------
        ids : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/private/dashboards/delete-batch",
            method="POST",
            json={
                "ids": ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import DashboardUpdatePublicScope, DashboardUpdatePublicType, DashboardWriteScope, DashboardWriteType

__all__ = ["DashboardUpdatePublicScope", "DashboardUpdatePublicType", "DashboardWriteScope", "DashboardWriteType"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .chat_completion_choice import ChatCompletionChoice
from .usage import Usage


class ChatCompletionResponse(UniversalBaseModel):
    id: typing.Optional[str] = None
    created: typing.Optional[int] = None
    model: typing.Optional[str] = None
    choices: typing.Optional[typing.List[ChatCompletionChoice]] = None
    usage: typing.Optional[Usage] = None
    system_fingerprint: typing.Optional[str] = None
    service_tier: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""Runner activation — called after the user's module loads to start the in-process loop."""

import logging
import os
import threading

from rich.console import Console
from rich.text import Text

from .. import Opik
from ..rest_api.types.agent import Agent
from ..rest_api.types.param import Param
from . import registry
from .in_process_loop import InProcessRunnerLoop

LOGGER = logging.getLogger(__name__)


def activate_runner() -> None:
    runner_mode = os.environ.get("OPIK_RUNNER_MODE")
    if runner_mode != "true":
        return

    runner_id = os.environ.get("OPIK_RUNNER_ID", "")
    project_name = os.environ.get("OPIK_PROJECT_NAME", "")

    if not runner_id:
        LOGGER.error("OPIK_RUNNER_ID not set, cannot activate runner")
        return

    _print_banner(runner_id, project_name)

    from . import prefixed_output

    prefixed_output.install()

    client = Opik(_show_misconfiguration_message=False)
    api = client.rest_client

    entrypoints = registry.get_all()
    if entrypoints:
        payload = {
            name: Agent(
                description=entry.get("docstring", ""),
                language="python",
                params=[
                    Param(name=p.name, type=p.type) for p in entry.get("params", [])
                ],
                timeout=0,
            ).dict()
            for name, entry in entrypoints.items()
        }
        api.runners.register_agents(runner_id, request=payload)

    LOGGER.info(
        "Runner activated with %d entrypoint(s): %s",
        len(entrypoints),
        ", ".join(entrypoints.keys()) if entrypoints else "(none)",
    )

    shutdown_event = threading.Event()
    loop = InProcessRunnerLoop(api, runner_id, shutdown_event)

    try:
        loop.run()
    finally:
        client.end()


def _print_banner(runner_id: str, project_name: str) -> None:
    console = Console()

    info = Text()
    info.append("   ")
    info.append("\u2800\u20dd", style="rgb(224,62,45)")
    info.append("opik  ", style="bold")
    info.append(f"runner: {runner_id}", style="dim")
    if project_name:
        info.append(f"  project: {project_name}", style="dim")
    console.print(info)
    console.print()



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .dashboard_update_public_scope import DashboardUpdatePublicScope
from .dashboard_update_public_type import DashboardUpdatePublicType
from .dashboard_write_scope import DashboardWriteScope
from .dashboard_write_type import DashboardWriteType

__all__ = ["DashboardUpdatePublicScope", "DashboardUpdatePublicType", "DashboardWriteScope", "DashboardWriteType"]



from collections import defaultdict
from typing import Dict, List, Optional, Tuple

from rich import align, console, panel, table, text


from opik.api_objects.dataset.evaluation_suite import types as suite_types

from . import test_result, evaluation_result
from .metrics import score_result


def _format_time(seconds: float) -> str:
    hours, remainder = divmod(seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    return f"{int(hours):02d}:{int(minutes):02d}:{int(seconds):02d}"


def _compute_average_scores(
    test_results: List[test_result.TestResult],
) -> Tuple[Dict[str, str], Dict[str, int]]:
    # Calculate average scores
    score_totals: Dict[str, float] = defaultdict(float)
    score_counts: Dict[str, int] = defaultdict(int)
    score_failed: Dict[str, int] = defaultdict(int)

    for result in test_results:
        for score in result.score_results:
            if score.scoring_failed:
                score_failed[score.name] += 1
                score_counts[score.name] += 0
            else:
                score_totals[score.name] += score.value
                score_counts[score.name] += 1
                score_failed[score.name] += 0

    average_scores = {
        name: "None" if counts == 0 else f"{score_totals[name] / counts:.4f}"
        for name, counts in score_counts.items()
    }
    failed_scores = {name: score_failed[name] for name in score_counts.keys()}

    return average_scores, failed_scores


def display_experiment_results(
    dataset_name: str,
    total_time: float,
    test_results: List[test_result.TestResult],
    experiment_scores: Optional[List[score_result.ScoreResult]] = None,
) -> None:
    average_scores, failed_scores = _compute_average_scores(test_results)
    nb_runs = len(test_results)

    # Count unique dataset items (not runs)
    unique_item_ids = {
        result.test_case.dataset_item_id
        for result in test_results
        if result.test_case.dataset_item_id is not None
    }
    nb_items = len(unique_item_ids) if unique_item_ids else nb_runs

    time_text = text.Text(f"Total time:        {_format_time(total_time)}")
    time_text.stylize("bold", 0, 18)
    time_text = align.Align.left(time_text)

    # Show both items and runs if they differ
    if nb_runs != nb_items:
        nb_samples_text = text.Text(
            f"Number of items:   {nb_items:,} ({nb_runs:,} runs)"
        )
    else:
        nb_samples_text = text.Text(f"Number of samples: {nb_items:,}")
    nb_samples_text.stylize("bold", 0, 18)
    nb_samples_text = align.Align.left(nb_samples_text)

    # Create test results text
    score_strings = text.Text("")
    for name, avg_score in average_scores.items():
        score_strings += text.Text(f"{name}: {avg_score} (avg)", style="green bold")
        if failed_scores[name] > 0:
            score_strings += text.Text(f" - {failed_scores[name]} failed", style="red")
        score_strings += text.Text("\n")

    # Add experiment scores if available
    if experiment_scores:
        for score in experiment_scores:
            score_strings += text.Text(
                f"{score.name}: {score.value:.4f}", style="green bold"
            )
            score_strings += text.Text("\n")

    aligned_test_results = align.Align.left(score_strings)

    # Combine table, time text, and test results
    content = table.Table.grid()
    content.add_row(text.Text(""))  # Empty space
    content.add_row(time_text)
    content.add_row(nb_samples_text)
    content.add_row(text.Text(""))
    content.add_row(aligned_test_results)

    # Create panel with content inside
    if nb_runs != nb_items:
        panel_title = f"{dataset_name} ({nb_items} items, {nb_runs} runs)"
    else:
        panel_title = f"{dataset_name} ({nb_items} samples)"
    panel_content = panel.Panel(
        content,
        title=panel_title,
        title_align="left",
        expand=False,
    )

    # Display results
    console_container = console.Console()
    console_container.print(panel_content)
    console_container.print("Uploading results to Opik ... ")


def display_suite_results(
    suite_name: str,
    total_time: float,
    suite_result: suite_types.EvaluationSuiteResult,
    verbose: int = 2,
) -> None:
    test_results = [
        tr
        for item_result in suite_result.item_results.values()
        for tr in item_result.test_results
    ]
    nb_runs = len(test_results)
    nb_items = suite_result.items_total
    items_passed = suite_result.items_passed
    items_total = suite_result.items_total
    suite_passed = suite_result.all_items_passed

    assertion_passed_count: Dict[str, int] = defaultdict(int)
    assertion_total_count: Dict[str, int] = defaultdict(int)

    for tr in test_results:
        for score in tr.score_results:
            assertion_total_count[score.name] += 1
            score_passed = not score.scoring_failed and (
                (isinstance(score.value, bool) and score.value) or score.value == 1
            )
            if score_passed:
                assertion_passed_count[score.name] += 1

    # Build display
    time_text = text.Text(f"Total time:        {_format_time(total_time)}")
    time_text.stylize("bold", 0, 18)
    time_text = align.Align.left(time_text)

    nb_samples_text = text.Text(f"Number of items:   {nb_items:,} ({nb_runs:,} runs)")
    nb_samples_text.stylize("bold", 0, 18)
    nb_samples_text = align.Align.left(nb_samples_text)

    pass_rate = items_passed / items_total if items_total > 0 else None
    if items_total == 0:
        pass_style = "yellow bold"
        pass_label = "NO ITEMS"
    elif suite_passed:
        pass_style = "green bold"
        pass_label = "PASSED"
    else:
        pass_style = "red bold"
        pass_label = "FAILED"
    pass_text = text.Text(f"Suite result:      {pass_label}", style=pass_style)
    pass_text.stylize("bold", 0, 18)
    pass_text = align.Align.left(pass_text)

    items_text = text.Text(f"Items passed:      {items_passed}/{items_total}")
    items_text.stylize("bold", 0, 18)
    items_text = align.Align.left(items_text)

    rate_value = f"{pass_rate:.1%}" if pass_rate is not None else "N/A"
    rate_text = text.Text(f"Pass rate:         {rate_value}")
    rate_text.stylize("bold", 0, 18)
    rate_text = align.Align.left(rate_text)

    # Compute per-item timing averages
    task_times = [
        tr.task_execution_time
        for tr in test_results
        if tr.task_execution_time is not None
    ]
    scoring_times = [
        tr.scoring_time for tr in test_results if tr.scoring_time is not None
    ]

    content = table.Table.grid()
    content.add_row(text.Text(""))
    content.add_row(time_text)
    content.add_row(nb_samples_text)
    content.add_row(pass_text)
    content.add_row(items_text)
    content.add_row(rate_text)

    if task_times and scoring_times:
        avg_task = sum(task_times) / len(task_times)
        avg_scoring = sum(scoring_times) / len(scoring_times)
        avg_total = avg_task + avg_scoring

        avg_task_text = text.Text(f"Avg task time:     {avg_task:.2f}s")
        avg_task_text.stylize("bold", 0, 18)
        avg_scoring_text = text.Text(f"Avg scoring time:  {avg_scoring:.2f}s")
        avg_scoring_text.stylize("bold", 0, 18)
        avg_total_text = text.Text(f"Avg total time:    {avg_total:.2f}s")
        avg_total_text.stylize("bold", 0, 18)

        content.add_row(text.Text(""))
        content.add_row(align.Align.left(avg_task_text))
        content.add_row(align.Align.left(avg_scoring_text))
        content.add_row(align.Align.left(avg_total_text))

    if verbose >= 2 and assertion_total_count:
        # Sort by pass rate ascending (most failed first)
        sorted_assertions = sorted(
            assertion_total_count.keys(),
            key=lambda n: (
                assertion_passed_count[n] / assertion_total_count[n]
                if assertion_total_count[n] > 0
                else 0.0
            ),
        )

        score_strings = text.Text("")
        for name in sorted_assertions:
            passed = assertion_passed_count[name]
            total = assertion_total_count[name]
            rate = passed / total if total > 0 else 0.0
            style = "green bold" if passed == total else "red bold"
            score_strings += text.Text(
                f"{name}: {rate:.0%} passed ({passed}/{total})\n",
                style=style,
            )

        content.add_row(text.Text(""))
        content.add_row(align.Align.left(score_strings))

    panel_content = panel.Panel(
        content,
        title=f"{suite_name} ({nb_items} items, {nb_runs} runs)",
        title_align="left",
        expand=False,
    )

    console_container = console.Console()
    console_container.print(panel_content)
    console_container.print("Uploading results to Opik ... ")


def display_experiment_link(experiment_url: str) -> None:
    console_container = console.Console()

    console_container.print(
        f"View the results [link={experiment_url}]in your Opik dashboard[/link]."
    )


def display_evaluation_scores_statistics(
    dataset_name: str,
    evaluation_results: evaluation_result.EvaluationResult,
) -> None:
    """
    Displays evaluation scores statistics for a given dataset.

    The function generates a summary of evaluation scores including mean, max,
    min, and optionally standard deviation for each metric in the evaluation
    results. The summarized scores are formatted and presented in a table
    within a panel for user clarity.

    Args:
        dataset_name: Name of the dataset for which evaluation statistics are
            being displayed.
        evaluation_results: An object containing evaluation results with
            aggregated scores and statistical data.
    """
    aggregated_view = evaluation_results.aggregate_evaluation_scores()
    if not aggregated_view.aggregated_scores:
        return

    # Create a table for the statistics
    stats_table = table.Table()
    stats_table.add_column("Name", style="cyan", no_wrap=True)
    stats_table.add_column("Mean", justify="right", style="green")
    stats_table.add_column("Min", justify="right", style="yellow")
    stats_table.add_column("Max", justify="right", style="yellow")
    stats_table.add_column("Std", justify="right", style="magenta")

    # Add rows for each metric
    for name, stats in aggregated_view.aggregated_scores.items():
        std_value = f"{stats.std:.4f}" if stats.std is not None else "N/A"
        stats_table.add_row(
            name,
            f"{stats.mean:.4f}",
            f"{stats.min:.4f}",
            f"{stats.max:.4f}",
            std_value,
        )

    # Create a panel with the table inside
    panel_content = panel.Panel(
        stats_table,
        title=f"Evaluation statistics for {dataset_name}",
        title_align="left",
        expand=False,
    )

    # Display results
    console_container = console.Console()
    console_container.print(panel_content)
